import { Transform } from 'stream'
import { StreamOptions } from './common'

export default class ConverterStream extends Transform {
  private bigintBuffer: bigint = 0n

  constructor(
    private readonly converter: (value: string | number | bigint) => string,
    readonly options: StreamOptions,
  ) {
    super()
  }

  _transform(chunk: Buffer, _encoding: BufferEncoding, callback: Function) {
    let chunkStr = chunk.toString(this.options.encoding)
    // we might run out of allocation for bigints?
    this.bigintBuffer *= 10n ** BigInt(chunkStr.length)
    this.bigintBuffer += BigInt(chunkStr)
    callback()
  }

  _flush(callback: Function) {
    this.push(this.converter(this.bigintBuffer), this.options.encoding)
    callback()
  }
}
